#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include <gnome.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"

#include <libgnomevfs/gnome-vfs.h>

static char filename[256];

static GtkWidget * window;

static GtkTextView * textview1;
static GtkTextBuffer * textbuffer;


void open_file(char * file_name)
{
	GnomeVFSHandle * file_handle;
	GnomeVFSFileInfo * file_info;
	GnomeVFSFileSize bytes_read;
	gpointer buf;
	if (gnome_vfs_open(&file_handle, file_name, GNOME_VFS_OPEN_READ) != GNOME_VFS_OK)
	{
		GtkWidget * dialog;
		dialog = gtk_message_dialog_new(window, GTK_DIALOG_DESTROY_WITH_PARENT,
        	GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
			"Невозможно открыть файл %s", file_name);
		gtk_window_set_title (GTK_WINDOW(dialog), "Ошибка");
		gtk_dialog_run (GTK_DIALOG (dialog));		
		gtk_widget_destroy (dialog);
		return;
	}	
	strncpy(filename, file_name, 256);
	gtk_text_buffer_set_text(textbuffer, "", 0);
	file_info = gnome_vfs_file_info_new();	
	gnome_vfs_get_file_info_from_handle(file_handle, file_info, 
		GNOME_VFS_FILE_INFO_DEFAULT);
	buf = g_malloc(file_info->size);				
	gnome_vfs_read(file_handle, buf, file_info->size, &bytes_read);
	gtk_text_buffer_set_text(textbuffer, buf, bytes_read);
	g_free(buf);
	gnome_vfs_file_info_unref(file_info);
	gnome_vfs_close(file_handle);
	gtk_text_buffer_set_modified(textbuffer, FALSE);	
}

void save_file(char * file_name)
{
	GnomeVFSHandle * file_handle;
	GnomeVFSFileSize bytes_read;
	GtkTextIter start, end;
	gchar * buf;
	if (gnome_vfs_create(&file_handle, file_name, GNOME_VFS_OPEN_WRITE, FALSE, 0666) 
		!= GNOME_VFS_OK)
	{
		GtkWidget * dialog;
		dialog = gtk_message_dialog_new(window, GTK_DIALOG_DESTROY_WITH_PARENT,
        	GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE,
			"Невозможно открыть файл %s", file_name);
		gtk_window_set_title (GTK_WINDOW(dialog), "Ошибка");
		gtk_dialog_run (GTK_DIALOG (dialog));		
		gtk_widget_destroy (dialog);
		return;
	}
	gtk_text_buffer_get_start_iter(textbuffer, &start);
	gtk_text_buffer_get_end_iter(textbuffer, &end);
	buf = gtk_text_buffer_get_text(textbuffer, &start, &end, FALSE);
	gnome_vfs_write(file_handle, buf, strlen(buf), &bytes_read);
	gnome_vfs_close(file_handle);	
	g_free(buf);
	gtk_text_buffer_set_modified(textbuffer, FALSE);
}

void
on_new1_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	if (gtk_text_buffer_get_modified(textbuffer))
	{
		GtkWidget * dialog;
		dialog = gtk_message_dialog_new(window, GTK_DIALOG_DESTROY_WITH_PARENT,
        	GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO,
			"Сохранить изменения в %s?", filename);
		gtk_window_set_title (GTK_WINDOW(dialog), "Файл был изменен");
   		if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_YES)
			on_save1_activate(menuitem, user_data);		
		gtk_widget_destroy (dialog);
	}
	gtk_text_buffer_set_text(textbuffer, "", 0);
	filename[0] = 0;
	gtk_text_buffer_set_modified(textbuffer, FALSE);
}


void
on_open1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget * dialog;
	if (gtk_text_buffer_get_modified(textbuffer))
	{
		GtkWidget * dialog;
		dialog = gtk_message_dialog_new(window, GTK_DIALOG_DESTROY_WITH_PARENT,
        	GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO,
			"Сохранить изменения в %s?", filename);
		gtk_window_set_title (GTK_WINDOW(dialog), "Файл был изменен");
   		if (gtk_dialog_run (GTK_DIALOG (dialog)) == GTK_RESPONSE_YES)
			on_save1_activate(menuitem, NULL);		
		gtk_widget_destroy (dialog);
	}	
	dialog = gtk_file_chooser_dialog_new("Открыть файл", window,
		GTK_FILE_CHOOSER_ACTION_OPEN,
		GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
		GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT, NULL);
	if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT)
	{
		char * file_name;
	    file_name = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
    	open_file(file_name);
		g_free (file_name);
	}
	gtk_widget_destroy(dialog);
}


void
on_save1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	if(filename[0] != 0)
		save_file(filename);
	else
		on_save_as1_activate(menuitem, user_data);
}


void
on_save_as1_activate                   (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkWidget * dialog;
	dialog = gtk_file_chooser_dialog_new ("Сохранить как...", window,
		GTK_FILE_CHOOSER_ACTION_SAVE,
		GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
		GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT,  NULL);
	gtk_file_chooser_set_do_overwrite_confirmation (GTK_FILE_CHOOSER (dialog), TRUE);
	if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_ACCEPT)
	{
		char * file_name;
	    file_name = gtk_file_chooser_get_filename (GTK_FILE_CHOOSER (dialog));
    	save_file(file_name);
		strncpy(filename, file_name, 256);
		g_free(file_name);
	}
	gtk_widget_destroy(dialog);
}

void
on_quit1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	if (!on_window1_delete_event(window, NULL, NULL))
		gtk_widget_destroy(window);				
}

void
on_cut1_activate                       (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkClipboard * cb = gtk_clipboard_get(gdk_atom_name("CLIPBOARD"));
	gtk_text_buffer_cut_clipboard(textbuffer, cb, TRUE);
}

void
on_copy1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	char * atom = gdk_atom_name("CLIPBOARD");
	GtkClipboard * cb = gtk_clipboard_get(atom);
	g_free(atom);
	gtk_text_buffer_copy_clipboard(textbuffer, cb);
}


void
on_paste1_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	char * atom = gdk_atom_name("CLIPBOARD");
	GtkClipboard * cb = gtk_clipboard_get(gdk_atom_name("CLIPBOARD"));
	g_free(atom);
	gtk_text_buffer_paste_clipboard(textbuffer, cb, NULL, TRUE);
}


void
on_clear1_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_properties1_activate                (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_preferences1_activate               (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_about1_activate                     (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{

}


void
on_window1_destroy                     (GtkObject       *object,
                                        gpointer         user_data)
{
	gtk_main_quit();
}

void
on_fontselect_activate                 (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
	GtkFontSelectionDialog * dialog = gtk_font_selection_dialog_new("Выбор шрифта");
	if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_OK)
	{
		char * font_name = gtk_font_selection_dialog_get_font_name(dialog);
		PangoFontDescription * font_desc = pango_font_description_from_string(font_name);	
		gtk_widget_modify_font (textview1, font_desc);
  		pango_font_description_free (font_desc);
		g_free(font_name);
	}
	gtk_widget_destroy(dialog);
}

gboolean
on_window1_delete_event                (GtkWidget       *widget,
                                        GdkEvent        *event,
                                        gpointer         user_data)
{
	if (gtk_text_buffer_get_modified(textbuffer))
	{
		int res;
		GtkWidget * dialog;
		dialog = gtk_message_dialog_new(window, GTK_DIALOG_DESTROY_WITH_PARENT,
        	GTK_MESSAGE_QUESTION, GTK_BUTTONS_NONE,
			"Сохранить изменения в %s?", filename);
		gtk_dialog_add_buttons(dialog, GTK_STOCK_YES, GTK_RESPONSE_YES, GTK_STOCK_NO, 
					GTK_RESPONSE_NO, "От_мена", GTK_RESPONSE_CANCEL, NULL); 
		gtk_window_set_title (GTK_WINDOW(dialog), "Файл был изменен");
   		res = gtk_dialog_run (GTK_DIALOG (dialog));
		gtk_widget_destroy (dialog);
		switch(res)							
		{
			case GTK_RESPONSE_YES:
				on_save1_activate(widget, user_data);
				break;
			case GTK_RESPONSE_CANCEL:
				return TRUE;
						
		 	default:;
		}
	}
  return FALSE;
}

void
on_window1_show                        (GtkWidget       *widget,
                                        gpointer         user_data)
{
	window = widget;
	textview1 = lookup_widget(window, "textview1");
	textbuffer = gtk_text_view_get_buffer(textview1);
}
